package se.gawell.gmbroadcast.ampq;

import java.io.IOException;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;

import se.gawell.gmbroadcast.BroadcastException;
import se.gawell.gmbroadcast.Consumer;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.QueueingConsumer;

public class AMPQConsumer implements Consumer {

	private final ObjectMapper objectMapper = new ObjectMapper();

	@Override
	public <MessageType> MessageType blockingReceive(Class<? extends MessageType> messageType) {
		try {
			QueueingConsumer consumer = initQueue(messageType);
			
			return receiveMessage(messageType, consumer);
		} catch (Exception e) {
			throw new BroadcastException(e);
		}
	}

	private <MessageType> QueueingConsumer initQueue(Class<? extends MessageType> messageType) throws IOException {
		ConnectionFactory factory = new ConnectionFactory();
		factory.setHost("localhost");
		Connection connection;
		connection = factory.newConnection();
		Channel channel = connection.createChannel();

		String queueName = messageType.getCanonicalName();
		channel.queueDeclare(queueName, false, false,
				false, null);
		QueueingConsumer consumer = new QueueingConsumer(channel);
		channel.basicConsume(queueName, true, consumer);
		return consumer;
	}

	private <MessageType> MessageType receiveMessage(Class<? extends MessageType> messageType, QueueingConsumer consumer)
			throws InterruptedException, IOException, JsonParseException, JsonMappingException {
		QueueingConsumer.Delivery delivery = consumer.nextDelivery();
		
		return (MessageType) objectMapper.readValue(delivery.getBody(), messageType);
	}

}
