package se.gawell.gmbroadcast.ampq;

import java.io.IOException;

import org.codehaus.jackson.JsonGenerationException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;

import se.gawell.gmbroadcast.BroadcastException;
import se.gawell.gmbroadcast.Producer;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;

public class AMPQProducer<MessageType> implements Producer<MessageType> {

	ObjectMapper objectMapper = new ObjectMapper();

	@Override
	public void broadcast(MessageType message) {
		try {
			String queueName = message.getClass().getCanonicalName();
			Channel channel = initializeQueue(queueName);
			byte[] serializedMessage = serializeMessage(message);
			sendMessage(queueName, channel, serializedMessage);
		} catch (IOException e) {
			throw new BroadcastException(e);
		}
	}

	private Channel initializeQueue(String queueName) throws IOException {
		ConnectionFactory factory = new ConnectionFactory();
		factory.setHost("localhost");
		Connection connection;
		connection = factory.newConnection();
		Channel channel = connection.createChannel();
		channel.queueDeclare(queueName, false, false, false, null);
		return channel;
	}
	
	private byte[] serializeMessage(MessageType message) throws IOException, JsonGenerationException,
	JsonMappingException {
		byte[] serializedMessage = objectMapper.writeValueAsBytes(message);
		return serializedMessage;
	}
	
	private void sendMessage(String queueName, Channel channel, byte[] serializedMessage) throws IOException {
		channel.basicPublish("", queueName, null, serializedMessage);
	}

}
